Package: biomaRt
Title: Interface to BioMart databases (i.e. Ensembl)
Version: 2.68.0
Authors@R: c(
    person("Steffen", "Durinck", , "biomartdev@gmail.com", role = "aut"),
    person("Wolfgang", "Huber", role = "aut"),
    person("Sean", "Davis", , "sdavis2@mail.nih.gov", role = "ctb"),
    person("Francois", "Pepin", role = "ctb"),
    person("Vince S", "Buffalo", role = "ctb"),
    person("Mike", "Smith", , "grimbough@gmail.com", role = "ctb",
           comment = c(ORCID = "0000-0002-7800-3848")),
    person("Hugo", "Gruson", , "hugo.gruson@embl.de", role = c("ctb", "cre"),
           comment = c(ORCID = "0000-0002-4094-1476")),
    person("German Network for Bioinformatics Infrastructure - de.NBI", role = "fnd")
  )
Description: In recent years a wealth of biological data has become
    available in public data repositories. Easy access to these valuable
    data resources and firm integration with data analysis is needed for
    comprehensive bioinformatics data analysis.  biomaRt provides an
    interface to a growing collection of databases implementing the
    BioMart software suite
    (<https://www.ensembl.org/info/data/biomart/index.html>). The package
    enables retrieval of large amounts of data in a uniform way without
    the need to know the underlying database schemas or write complex SQL
    queries.  The most prominent examples of BioMart databases are
    maintained by Ensembl, which provides biomaRt users direct access to a
    diverse set of data and enables a wide range of powerful online
    queries from gene annotation to database mining.
License: Artistic-2.0
URL: https://github.com/Huber-group-EMBL/biomaRt,
        https://huber-group-embl.github.io/biomaRt/
BugReports: https://github.com/Huber-group-EMBL/biomaRt/issues
Depends: methods, R (>= 4.5.0)
Imports: AnnotationDbi, BiocFileCache, curl, httr2, progress, stringr,
        utils, xml2
Suggests: BiocStyle, httptest2, knitr, mockery, rmarkdown, testthat (>=
        3.0.0), withr
VignetteBuilder: knitr
biocViews: Annotation
Config/Needs/website: xfun
Config/testthat/edition: 3
Encoding: UTF-8
LazyLoad: yes
NeedsCompilation: no
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Collate: 'biomaRt-package.R' 'biomaRtClasses.R' 'methods-Mart.R'
        'biomaRt.R' 'caching.R' 'ensembl.R' 'ensembl_wrappers.R'
        'ensembl_ssl_settings.R' 'utilityFunctions.R'
        'non-biomart-utils.R' 'exportFASTA.R' 'NP2009code.R'
git_url: https://git.bioconductor.org/packages/biomaRt
git_branch: RELEASE_3_23
git_last_commit: ff93209
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
Packaged: 2026-04-28 20:57:08 UTC; biocbuild
Author: Steffen Durinck [aut],
  Wolfgang Huber [aut],
  Sean Davis [ctb],
  Francois Pepin [ctb],
  Vince S Buffalo [ctb],
  Mike Smith [ctb] (ORCID: <https://orcid.org/0000-0002-7800-3848>),
  Hugo Gruson [ctb, cre] (ORCID: <https://orcid.org/0000-0002-4094-1476>),
  German Network for Bioinformatics Infrastructure - de.NBI [fnd]
Maintainer: Hugo Gruson <hugo.gruson@embl.de>
