# biomaRt <img src="man/figures/logo.svg" align="right" width="120" />

The package provides an R interface to datasets that provide access via the
BioMart software suite.

## Current Status

| GitHub Actions        | BioC           | Test Coverage |
| ------------- |-------------| -----|
| [![R-CMD-check](https://github.com/Huber-group-EMBL/biomaRt/actions/workflows/main.yml/badge.svg)](https://github.com/Huber-group-EMBL/biomaRt/actions/workflows/main.yml) | [![BioC Status](https://bioconductor.org/shields/build/devel/bioc/biomaRt.svg)](http://bioconductor.org/checkResults/devel/bioc-LATEST/biomaRt/) | [![Codecov](http://img.shields.io/codecov/c/github/grimbough/biomaRt.svg)](https://codecov.io/gh/grimbough/biomaRt) |

## Contact

For bug reports, please register an [issue](https://github.com/grimbough/biomaRt/issues) here on Github. For usage queries please post a question on the [Bioconductor Support Forum](https://support.bioconductor.org/p/new/post/?tag_val=biomaRt).

## Funding

Funding for continued development and maintenance of this package is provided by the German Network for Bioinformatics Infrastructure

<a href="https://www.denbi.de"><img src="https://tess.elixir-europe.org/system/content_providers/images/000/000/063/original/deNBI_Logo_rgb.jpg" width="400" align="left"></a>