#' Deprecated and defunct functions in package \pkg{biomaRt}
#'
#' These functions have been removed from biomaRt and replaced with
#' alternatives.
#'
#'
#' The following functions are defunct and no longer work; use the replacement
#' indicated below:
#'
#' * filterOptions: [listFilterOptions()]
#' * listFilterValues: [listFilterOptions()]
#' * searchFilterValues: [searchFilterOptions()]
#'
#' @name biomaRt-deprecated
#' @aliases filterOptions searchFilterValues listFilterValues
NULL

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import methods
## usethis namespace: end
NULL
